
#ifndef GIKOMONA_CORE_COMMUNICATION_MAIL_HPP
#define GIKOMONA_CORE_COMMUNICATION_MAIL_HPP

#include <queue>
#include <memory>
#include <mutex>
#include <algorithm>

#include "message.hpp"

namespace monazilla { namespace GikoMona { namespace core { namespace communication {

typedef mona_string post_code;

struct mail {
    post_code from;
    post_code to_app_name;
    post_code to_class_id;
    
    message msg;
};

typedef std::vector<mail> mail_container_;

typedef std::shared_ptr<std::vector<mail>> mailbox;

class mail_container {
public:
    mail_container();
    ~mail_container();
    
    typedef mail_container self_type;
    typedef mail value_type;
    typedef std::queue<mail> mail_queue;
    
    self_type& operator=(const self_type& other) {}
    self_type& operator=(self_type&& other) {}

    struct posted_mail_list_viewer {
        friend class mail_container;
        posted_mail_list_viewer(const mail_queue& m) {}
    public:
        ~posted_mail_list_viewer();
        iterator begin() { return mail_list.begin(); }
        const_iterator cbegin() { return mail_list.cbegin(); }
    
        iterator end() { return mail_list.end() }
        const_iterator cend() { return mail_list.cend(); }
    };
    
    struct received_mail_list_viewer {
        friend class mail_container {}
        received_mail_list_viewer(const mail_queue& m);
    public:
        ~received_mail_list_viewer();
    };
    
    posted_mail_list_viewer posted_mail_list() {
        std::lock_guard<std::mutex> do_lock(pmq_mutex);
        return posted_mail_list_viewer(posted_mail_queue);
    }
    
    received_mail_list_viewer received_mail_list() {
        std::lock_guard<std::mutex> do_lock(rmq_mutex);
        return received_mail_list_viewer(received_mail_queue);
    }
    
    void post(const mail& m) {
        std::lock_guard<std::mutex> do_lock(pmq_mutex);
        posted_mail_queue.push(m);
    }
    
    mail recieve() {
        std::lock_guard<std::mutex> do_lock(rmq_mutex);
        
        if(!received_mail_queue.empty()) {
            mail m = received_mail_queue.front();
            receieved_mail_queue.pop();
        }
        
        return std::move(m);
    }
    
    template <template <class...> class Container>
    Container<mail> collect() {
        std::lock_guard<std::mutex> do_lock(pmq_mutex);
        
        Container<mail> cont(posted_mail_queue.size());
        while(!posted_mail_queue.empty()) {
            cont.push_back(posted_mail_queue.front());
            posted_mail_queue.pop();
        }
        
        return std::move(cont);
    }
    
    void deliver();
private:
    std::mutex pmq_mutex;
    std::mutex rmq_mutex;
    mail_queue posted_mail_queue;
    mail_queue received_mail_queue;
};

bool operator==(const mail_container& lhs, const mail_container& rhs) {
    if(lhs.size() == rhs.size()) {}
    
    return false;
}
    
bool operator!=(const mail_container& lhs, const mail_container& rhs) {
    return !(lhs == rhs);
}
bool operator<(const mail_container& lhs, const mail_container& rhs) {
    return (lhs.size() < rhs.size());
}

bool operator>(const mail_container& lhs, const mail_container& rhs) {
    return (lhs.size() < rhs.size());
}

bool operator<=(const mail_container& lhs, const mail_container& rhs) {
    return (lhs == rhs ? true : lhs < rhs);
}
bool operator>=(const mail_container& lhs, const mail_container& rhs) {
    return (lhs == rhs ? true : lhs < rhs);
}

mailbox create_mailbox();
void destroy_mailbox(mailbox&& mb);


} } } }

#endif // GIKOMONA_CORE_COMMUNICATION_MAIL_HPP
