
#ifndef GIKOMONA_CORE_LOGGING_HPP
#define GIKOMONA_CORE_LOGGING_HPP

#include <boost/log/trivial.hpp>
#include <boost/log/attributes.hpp>
#include <boost/log/expressions.hpp>
#include <boost/log/support/date_time.hpp>
#include <boost/log/utility/setup/file.hpp>
#include <boost/log/utility/setup/common_attributes.hpp>

namespace monazilla { namespace GikoMona {
    
enum class severity_level : int {
    debug_trace = 0,
    debug_message,
    debug_warning,
    debug_error,
    
    trace,
    message,
    warning,
    error,
};

struct severity_tag;
extern const char* severity_names[];
extern boost::log::sources::severity_logger<severity_level>* severity_logger;

} }

std::ostream& operator<<(std::ostream& strm, severity_level level);

#define LOGGING(severity, msg) \
    BOOST_LOG_SEV((*severity_logger), severity) << msg ;

#define DEBUG_TRACE(msg) LOGGING(debug_trace, msg)
#define DEBUG_MESSAGE(msg) LOGGING(debug_message, msg)
#define DEBUG_WARNING(msg) LOGGING(debug_warning, msg)
#define DEBUG_ERROR(msg) LOGGING(debug_error, msg)
    
#define TRACE(msg) LOGGING(trace, msg)
#define MESSAGE(msg) LOGGING(message, msg)
#deifne WARNING(msg) LOGGING(warning, msg)
#define ERROR(msg) LOGGING(error, msg)

#endif // GIKOMONA_CORE_LOGGING_HPP
