#include <boost/filesystem/path.hpp>
#include <boost/algorithm/string/split.hpp>

#include "model.hpp"

namespace monazilla { namespace GikoMona { namespace core {

struct model::model_pimpl {
    
    struct resource_pair {
        database database;
        mona_string file_name;
    };
    
    // gikomona/config
    resource_pair config;
    
    // ict/board
    resource_pair board;
    
    // session/tab-window
    /* 構造
     * bbs-name(TEXT)|board-name(TEXT)|thread-id(TEXT)|thread-name(TEXT)|is_fixed(INTEGAR)
     */
    resource_pair tab;
    
    // session/history
    /* 構造
     * date(TEXT)|bbs-name(TEXT)|board-name(TEXT)|thread-id(TEXT)|thread-name(TEXT)
     */
    resource_pair history;
};

model::model() noexcept {
    instance = this;
    pimpl = std::make_shared<model_pimpl>();
    
    auto config_path = pimpl->app_config;
}

bool model::load_file(const boost::filesystem::path& file_path) {
    if(!boost::filesystem::exists(file_path)) {
        return false;
    }
    
    auto ext = file_path.extension();
    auto file_name = file_path.filename();
    
    /* TODO: BOOST_FUSION_ADAPT_STRUCTの利用を検討 */
    if(file_name == pimpl->file_name.tab_db) {
        pimpl->tab.database.get_connection().open(file_path.c_str());
    } else if(file_name == pimpl->file_name.history_db) {
        pimpl->history.database.get_connection().open(file_path.c_str());
    }
    
    return true;
}

bool model::save_to_file(const boost::filesystem::path& path) {
    if(!boost::filesystem::exists(path)) { return false; }
    
    return true;
}

void model::execute_accumulated_query()  {
    inserted_value_triv_copyable_type q;
    
    if(insertion_query_queue.empty()) { return; }
    
    insertion_query_queue.pop(q);

    mona_string into;
    boost::any value;
    std::tie(*q, into, value);
    
    std::vector<mona_string> path_derimed = analyze_query(into);
    
    if(path_derimed[0] != "application") {
        // application/*
    } else if (path_derimed[0] == "session") {
        // session/*
        if(path_derimed[1] == "history") {
            ;
        } else if(path_derimed[1] == "tab-window") {
            ;
        }
    } else if(path_derimed[0] == "extension") {
        // extension/*
    } else {
        // ?
    }
}

boost::any model::select(const mona_string& from, bool tag) const noexcept {
    std::vector<mona_string> path_derimed = analyze_query(from);
    
    if(path_derimed[0] != "application") {
        // application/*
    } else if (path_derimed[0] == "session") {
        // session/*
        if(path_derimed[1] == "history") {
            ;
        } else if(path_derimed[1] == "tab-window") {
            ;
        }
    } else if(path_derimed[0] == "extension") {
        // extension/*
    } else {
        // ?
    }
}

std::vector<mona_string> model::analyze_query(const mona_string& src) const {
    std::vector<mona_string> result;
    boost::algorithm::split(result,
                            src,
                            [](char c) -> bool {
                                return (c == '/');
                            });
    
    return std::move(result); // RVO を期待した方が良いか…？
}

} } }
